#!/bin/awk -f
## Shift time-stamps by specified offset
## 
## Parameter: dt - time offset in seconds
##
## Description: replace time stamps in the mass-file (and stm-file obligatory!)
## with those shifted by dt. This is normally needed when the Turbina time
## was wrong (say, by +/- 3600 sec) during observations causing the wrong
## values of the airmass and incorrect results referencing.
##
## Other function is the conversion from the old-format data to new (inserting
## the date as the second field and modification of UT>=24 by -=24).
##
## Note: start with dt=0 or without parameters makes output identical to input 
##
## Use as: dtime.awk [<]<mass-or-stm-file> [dt=val] [date=yymmdd] >result-file
##  where date is the strictly 6-symbols EVENING date for the old-format 
##  mass-files (normally taken from their name, e.g. 020304 for 4 March 2002).
BEGIN{
	dt = 0
	date = 0
	fmtrep = 0
}

NR==1 {
	if (dt!=0) print "## ==== DTIME CORRECT: by "dt" sec"
}

substr($2,3,1)==":" && !fmtrep && date!=0 {
	print "## ==== DTIME FORMAT CORRECTION: insert date "date
	# reformat date from yymmdd to YYYYmmdd:
	date = "20" substr(date,1,2) "-" substr(date,3,2) "-" substr(date,5,2)
	fmtrep = 1
}

substr($2,3,1)==":" && substr($2,6,1)==":" && date!=0 {
	$0 = substr($0,1,2) date " " substr($0,3)
	if ($1=="I" || $1=="J") {
		NF++
		$(NF) = $(NF-1)
		$(NF-1) = date
	}
}

substr($2,5,1)=="-" && substr($2,8,1)=="-" && (dt!=0 || length(date)==10) {
	day = gensub("-"," ","G",$2)
	ut = gensub(":"," ","G",$3)
	daytime = day" "ut
	t = mktime(daytime) + dt
	$2 = strftime("%Y-%m-%d",t);
	$3 = strftime("%H:%M:%S",t);
	if ($1=="I" || $1=="J") {
		day2 = gensub("-"," ","G",$(NF-1))
		ut2 = gensub(":"," ","G",$(NF))
		daytime2 = day2" "ut2
		t = mktime(daytime2) + dt
		$(NF-1) = strftime("%Y-%m-%d",t);
		$(NF) = strftime("%H:%M:%S",t);
	}
}

{ print }
